// This class was created by JaTiTV.

package net.t2code.t2codelib.SPIGOT.api.yaml;

import net.t2code.t2codelib.T2CconfigItem;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class T2CconfigWriter {

    private static FileConfiguration config;


    public static void createConfig(File configFile, T2CconfigItem[] values, String... header) {
        boolean exist = configFile.exists();
        if (!exist) {
            configFile.getParentFile().mkdirs();
            try {
                configFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
                return;
            }
        } else exist = true;

        config = YamlConfiguration.loadConfiguration(configFile);
        Map<String, List<String>> comments = new LinkedHashMap<>();

        for (T2CconfigItem item : values) {
            if (item.getForceSet() || !exist) {
                config.addDefault(item.getPath(), item.getValue());
                comments.put(item.getPath(), item.getComments());
            }
        }

        // Copy default values if they are missing
        config.options().copyDefaults(true);

        saveConfigWithComments(configFile, comments, header);
        readConfig(config, values);
    }

    private static void readConfig(FileConfiguration config, T2CconfigItem[] values) {
        for (T2CconfigItem value : values) {
            value.setValue(config.get(value.getPath()));
        }
    }

    private static void saveConfigWithComments(File file, Map<String, List<String>> comments, String... headers) {
        try {
            StringBuilder configContent = new StringBuilder();
            for (String h : headers) {
                configContent.append(h).append("\n");
            }
            configContent.append("\n");
            addSection(config, comments, configContent, "", 0);

            // Write the content to the file
            Files.write(file.toPath(), configContent.toString().getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void addSection(ConfigurationSection section, Map<String, List<String>> comments, StringBuilder builder, String prefix, int indentLevel) {
        String indent = "  ".repeat(indentLevel);


        for (String key : section.getKeys(false)) {
            String fullKey = prefix.isEmpty() ? key : prefix + "." + key;
            Object value = section.get(key);

            // Add comment if it exists for this key
            List<String> commentList = comments.get(fullKey);
            if (commentList != null) {
                for (String c : commentList) {
                    builder.append(indent).append("# ").append(c).append("\n");
                }
            }

            // Check if the value is a section (nested map)
            if (value instanceof ConfigurationSection) {
                // Correctly add the section
                builder.append(indent).append(key).append(":\n");
                addSection((ConfigurationSection) value, comments, builder, fullKey, indentLevel + 1);
            } else {
                // Add value with proper indentation
                if (value instanceof List) {
                    builder.append(indent).append(key).append(": ").append("\n");
                    List<Object> zw = (List<Object>) value;
                    for (Object s : zw) {
                        if (s instanceof String) {
                            builder.append(indent).append("- \"").append(s).append("\"\n");
                        } else builder.append(indent).append("- ").append(s).append("\n");
                    }
                } else {
                    if (value instanceof String) {
                        builder.append(indent).append(key).append(": \"").append(value).append("\"\n");
                    } else builder.append(indent).append(key).append(": ").append(value).append("\n");
                }
            }
        }
    }
}
